@@ -1,41 +1,33 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2015 The Qt Company Ltd.
 ** Copyright (C) 2013 Ivan Komissarov.
-** Contact: http://www.qt-project.org/legal
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the DDS plugin in the Qt ImageFormats module.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -43,12 +35,14 @@
 #include "qddshandler.h"
 
 #include <QtCore/qdebug.h>
-#include <QtCore/qmath.h>
-
 #include <QtGui/qimage.h>
 
+#include <cmath>
+
 #include "ddsheader.h"
 
+#include <cmath>
+
 #ifndef QT_NO_DATASTREAM
 
 QT_BEGIN_NAMESPACE
@@ -109,7 +103,7 @@ struct FormatInfo
 static const FormatInfo formatInfos[] = {
     { FormatA8R8G8B8,    DDSPixelFormat::FlagRGBA, 32, 0x00ff0000, 0x0000ff00, 0x000000ff, 0xff000000 },
     { FormatX8R8G8B8,    DDSPixelFormat::FlagRGB,  32, 0x00ff0000, 0x0000ff00, 0x000000ff, 0x00000000 },
-    { FormatA2B10G10R10, DDSPixelFormat::FlagRGBA, 32, 0x000003ff, 0x0000fc00, 0x3ff00000, 0xc0000000 },
+    { FormatA2B10G10R10, DDSPixelFormat::FlagRGBA, 32, 0x000003ff, 0x000ffc00, 0x3ff00000, 0xc0000000 },
     { FormatA8B8G8R8,    DDSPixelFormat::FlagRGBA, 32, 0x000000ff, 0x0000ff00, 0x00ff0000, 0xff000000 },
     { FormatX8B8G8R8,    DDSPixelFormat::FlagRGB,  32, 0x000000ff, 0x0000ff00, 0x00ff0000, 0x00000000 },
     { FormatG16R16,      DDSPixelFormat::FlagRGBA, 32, 0x0000ffff, 0xffff0000, 0x00000000, 0x00000000 },
@@ -166,6 +160,101 @@ static const Format knownFourCCs[] = {
 };
 static const size_t knownFourCCsSize = sizeof(knownFourCCs)/sizeof(Format);
 
+struct FormatName
+{
+    Format format;
+    const char *const name;
+};
+static const FormatName formatNames[] = {
+    { FormatUnknown,  "unknown" },
+
+    { FormatR8G8B8,   "R8G8B8"  },
+    { FormatA8R8G8B8, "A8R8G8B8" },
+    { FormatX8R8G8B8, "X8R8G8B8" },
+    { FormatR5G6B5,   "R5G6B5" },
+    { FormatX1R5G5B5, "X1R5G5B5" },
+    { FormatA1R5G5B5, "A1R5G5B5" },
+    { FormatA4R4G4B4, "A4R4G4B4" },
+    { FormatR3G3B2, "R3G3B2" },
+    { FormatA8, "A8" },
+    { FormatA8R3G3B2, "A8R3G3B2" },
+    { FormatX4R4G4B4, "X4R4G4B4" },
+    { FormatA2B10G10R10, "A2B10G10R10" },
+    { FormatA8B8G8R8, "A8B8G8R8" },
+    { FormatX8B8G8R8, "X8B8G8R8" },
+    { FormatG16R16, "G16R16" },
+    { FormatA2R10G10B10, "A2R10G10B10" },
+    { FormatA16B16G16R16, "A16B16G16R16" },
+
+    { FormatA8P8, "A8P8" },
+    { FormatP8, "P8" },
+
+    { FormatL8, "L8" },
+    { FormatA8L8, "A8L8" },
+    { FormatA4L4, "A4L4" },
+
+    { FormatV8U8, "V8U8" },
+    { FormatL6V5U5, "L6V5U5" },
+    { FormatX8L8V8U8, "X8L8V8U8" },
+    { FormatQ8W8V8U8, "Q8W8V8U8" },
+    { FormatV16U16, "V16U16" },
+    { FormatA2W10V10U10, "A2W10V10U10" },
+
+    { FormatUYVY, "UYVY" },
+    { FormatR8G8B8G8, "R8G8_B8G8" },
+    { FormatYUY2, "YUY2" },
+    { FormatG8R8G8B8, "G8R8_G8B8" },
+    { FormatDXT1, "DXT1" },
+    { FormatDXT2, "DXT2" },
+    { FormatDXT3, "DXT3" },
+    { FormatDXT4, "DXT4" },
+    { FormatDXT5, "DXT5" },
+    { FormatRXGB, "RXGB" },
+    { FormatATI2, "ATI2" },
+
+    { FormatD16Lockable, "D16Lockable" },
+    { FormatD32, "D32" },
+    { FormatD15S1, "D15S1" },
+    { FormatD24S8, "D24S8" },
+    { FormatD24X8, "D24X8" },
+    { FormatD24X4S4, "D24X4S4" },
+    { FormatD16, "D16" },
+
+    { FormatD32FLockable, "D32FLockable" },
+    { FormatD24FS8, "D24FS8" },
+
+    { FormatD32Lockable, "D32Lockable" },
+    { FormatS8Lockable, "S8Lockable" },
+
+    { FormatL16, "L16" },
+
+    { FormatVertexData, "VertexData" },
+    { FormatIndex32, "Index32" },
+    { FormatIndex32, "Index32" },
+
+    { FormatQ16W16V16U16, "Q16W16V16U16" },
+
+    { FormatMulti2ARGB8, "Multi2ARGB8" },
+
+    { FormatR16F, "R16F" },
+    { FormatG16R16F, "G16R16F" },
+    { FormatA16B16G16R16F, "A16B16G16R16F" },
+
+    { FormatR32F, "R32F" },
+    { FormatG32R32F, "G32R32F" },
+    { FormatA32B32G32R32F, "A32B32G32R32F" },
+
+    { FormatCxV8U8, "CxV8U8" },
+
+    { FormatA1, "A1" },
+    { FormatA2B10G10R10_XR_BIAS, "A2B10G10R10_XR_BIAS" },
+    { FormatBinaryBuffer, "BinaryBuffer" },
+
+    { FormatP4, "P4" },
+    { FormatA4P4, "A4P4" }
+};
+static const size_t formatNamesSize = sizeof(formatNames)/sizeof(FormatName);
+
 static inline int maskToShift(quint32 mask)
 {
     if (mask == 0)
@@ -236,7 +325,7 @@ static Format getFormat(const DDSHeader &dds)
             if ((format.flags & info.flags) == info.flags &&
                  format.rgbBitCount == info.bitCount &&
                  format.rBitMask == info.rBitMask &&
-                 format.bBitMask == info.bBitMask &&
+                 format.gBitMask == info.gBitMask &&
                  format.bBitMask == info.bBitMask &&
                  format.aBitMask == info.aBitMask) {
                 return info.format;
@@ -499,7 +588,7 @@ static QImage readATI2(QDataStream &s, quint32 width, quint32 height)
                     const double fx = nx / 127.5 - 1.0;
                     const double fy = ny / 127.5 - 1.0;
                     const double fxfy = 1.0 - fx * fx - fy * fy;
-                    const double fz = fxfy > 0 ? sqrt(fxfy) : -1.0;
+                    const double fz = fxfy > 0 ? std::sqrt(fxfy) : -1.0;
                     const quint8 nz = quint8((fz + 1.0) * 127.5);
 
                     line[j + l] = qRgb(nx, ny, nz);
@@ -577,9 +666,9 @@ static double readFloat16(QDataStream &s)
     quint16 fraction = value & 0x3FF;
 
     if (exp == 0)
-        return sign * qPow(2.0, -14.0) * fraction / 1024.0;
+        return sign * std::pow(2.0, -14.0) * fraction / 1024.0;
     else
-        return sign * qPow(2.0, exp - 15) * (1 + fraction / 1024.0);
+        return sign * std::pow(2.0, exp - 15) * (1 + fraction / 1024.0);
 }
 
 static inline float readFloat32(QDataStream &s)
@@ -725,7 +814,7 @@ static QImage readCxV8U8(QDataStream &s, const quint32 width, const quint32 heig
             const quint8 vn = v + 128, un = u + 128;
 
             const double vd = vn / 127.5 - 1.0, ud = un / 127.5 - 1.0;
-            const quint8 c = 255 * sqrt(1.0 - vd * vd - ud * ud);
+            const quint8 c = 255 * std::sqrt(1.0 - vd * vd - ud * ud);
             line[x] = qRgb(vn, un, c);
         }
     }
@@ -1256,7 +1345,29 @@ static QImage readCubeMap(QDataStream &s, const DDSHeader &dds, const int fmt)
     return image;
 }
 
+static QByteArray formatName(int format)
+{
+    for (size_t i = 0; i < formatNamesSize; ++i) {
+        if (formatNames[i].format == format)
+            return formatNames[i].name;
+    }
+
+    return formatNames[0].name;
+}
+
+static int formatByName(const QByteArray &name)
+{
+    const QByteArray loweredName = name.toLower();
+    for (size_t i = 0; i < formatNamesSize; ++i) {
+        if (QByteArray(formatNames[i].name).toLower() == loweredName)
+            return formatNames[i].format;
+    }
+
+    return FormatUnknown;
+}
+
 QDDSHandler::QDDSHandler() :
+    m_format(FormatA8R8G8B8),
     m_currentImage(0),
     m_scanState(ScanNotScanned)
 {
@@ -1303,6 +1414,11 @@ bool QDDSHandler::read(QImage *outImage)
 
 bool QDDSHandler::write(const QImage &outImage)
 {
+    if (m_format != FormatA8R8G8B8) {
+        qWarning() << "Format" << formatName(m_format) << "is not supported";
+        return false;
+    }
+
     const QImage image = outImage.convertToFormat(QImage::Format_ARGB32);
 
     QDataStream s(device());
@@ -1362,6 +1478,10 @@ QVariant QDDSHandler::option(QImageIOHandler::ImageOption option) const
     switch (option) {
     case QImageIOHandler::Size:
         return QSize(m_header.width, m_header.height);
+    case QImageIOHandler::SubType:
+        return formatName(m_format);
+    case QImageIOHandler::SupportedSubTypes:
+        return QVariant::fromValue(QList<QByteArray>() << formatName(FormatA8R8G8B8));
     default:
         break;
     }
@@ -1369,9 +1489,21 @@ QVariant QDDSHandler::option(QImageIOHandler::ImageOption option) const
     return QVariant();
 }
 
+void QDDSHandler::setOption(QImageIOHandler::ImageOption option, const QVariant &value)
+{
+    if (option == QImageIOHandler::SubType) {
+        const QByteArray subType = value.toByteArray();
+        m_format = formatByName(subType.toUpper());
+        if (m_format == FormatUnknown)
+            qWarning() << "unknown format" << subType;
+    }
+}
+
 bool QDDSHandler::supportsOption(QImageIOHandler::ImageOption option) const
 {
-    return option == QImageIOHandler::Size;
+    return (option == QImageIOHandler::Size)
+            || (option == QImageIOHandler::SubType)
+            || (option == QImageIOHandler::SupportedSubTypes);
 }
 
 int QDDSHandler::imageCount() const
@@ -1411,6 +1543,9 @@ bool QDDSHandler::ensureScanned() const
 
     m_scanState = ScanError;
 
+    QDDSHandler *that = const_cast<QDDSHandler *>(this);
+    that->m_format = FormatUnknown;
+
     if (device()->isSequential()) {
         qWarning() << "Sequential devices are not supported";
         return false;
@@ -1419,7 +1554,6 @@ bool QDDSHandler::ensureScanned() const
     qint64 oldPos = device()->pos();
     device()->seek(0);
 
-    QDDSHandler *that = const_cast<QDDSHandler *>(this);
     QDataStream s(device());
     s.setByteOrder(QDataStream::LittleEndian);
     s >> that->m_header;
